#include "phc.h"
#include "m33.h"
#include <math.h>

PHC::PHC(int _w, int _h, float hfov) :
 a(1.0f, 0.0f, 0.0f), b(0.0f, -1.0f, 0.0f), C(0.0f, 0.0f, 0.0f) ,
 w(_w), h(_h) {

  c[0] = -(float)w/2.0f;
  c[1] = (float)h/2.0f;
  float dhfov = hfov * 3.1415926535f / 180.0f;
  c[2] = c[0] / tanf(dhfov/2.0f);

}

M33 PHC::GetCamMat() {

  M33 ret;
  ret.SetColumn(0, a);
  ret.SetColumn(1, b);
  ret.SetColumn(2, c);
  return ret;

}

bool PHC::Project(V3 P, V3 &projP) {

  M33 camMat = GetCamMat();
  V3 q = camMat.Inverted()*(P-C);
  if (q[2] <= 0.001f) {
    return false;
  }

  projP = V3(q[0]/q[2], q[1]/q[2], q[2]);
  if (
    projP[0] < 0.0f || projP[0] >= (float)w ||
    projP[1] < 0.0f || projP[1] >= (float)h)
    return false;

  return true;

}

